#import "BarcodeDemoViewController.h"
#import "AsBarcodeItem.h"
#import "BarcodeDemoTableViewCell.h"
#import "UserData.h"
#import "CSVData.h"
#import "SystemSetting.h"
@interface BarcodeDemoViewController ()
{
    BOOL isScanning;
}
@property (weak, nonatomic) IBOutlet UITableView *tableView;
@property (weak, nonatomic) IBOutlet UIBarButtonItem *outputBtn;
@property (weak, nonatomic) IBOutlet UIPickerView *pickerView;
@property (weak, nonatomic) IBOutlet UIView *rescanTimePickerTopView;
@property (weak, nonatomic) IBOutlet UITextField *rescanTimeTextField;
@property (weak, nonatomic) IBOutlet UIButton *startScanBtn;
@property (weak, nonatomic) IBOutlet UILabel *totalCountLabel;
@property (strong, nonatomic) IBOutletCollection(UIControl) NSArray *toDisableElems;
@property (strong, nonatomic) NSArray *rescanTimePickerData;
@property (strong, nonatomic) NSArray *rescanTimeData;
@property (readwrite, nonatomic) NSInteger rescanTimeValue;
@property (strong, nonatomic) NSMutableArray *barcodeData;

@end

@implementation BarcodeDemoViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    self.rescanTimeTextField.text = [self.rescanTimePickerData objectAtIndex:[[SystemSetting sharedSystemSetting] getBarcodeReScanTime]];
    self.rescanTimeValue = [[SystemSetting sharedSystemSetting] getBarcodeReScanTime];
    if ([[SystemSetting sharedSystemSetting] getBarcodeData]) {
        self.barcodeData = [[NSMutableArray alloc] initWithArray:[[SystemSetting sharedSystemSetting] getBarcodeData]];
    }else{
        self.barcodeData = [[NSMutableArray alloc]init];
    }
    self.totalCountLabel.text = [NSString stringWithFormat:@"%lu", (unsigned long)self.barcodeData.count];
    [AppDelegate showGlobalProgressHUDWithTitle:@"Loading..."];
    appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
    [appDelegate.mReader setDelegate:self];
    if (appDelegate.mReader == nil) {
        return;
    }
}

- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
    NSNotificationCenter *center = [NSNotificationCenter defaultCenter];
    [center addObserver:self selector:@selector(AsReaderGUNDisconnected:) name:@"AsReaderGUNDisconnected" object:nil];
    [appDelegate.mReader setDelegate:self];
}

- (void)viewDidAppear:(BOOL)animated
{
//    appDelegate.mReader.isUseKeyAction = [UserData getIsUseKeyAction];
    [appDelegate.mReader setBarcodeMode:YES isKeyAction:YES];
    [AppDelegate dismissGlobalHUD];
    [appDelegate.mReader setScanMode:BarcodeScanMode];
}

- (void)viewWillDisappear:(BOOL)animated {
    [super viewWillDisappear:animated];
    [[NSNotificationCenter defaultCenter] removeObserver:self name:@"AsReaderGUNDisconnected" object:nil];
    [appDelegate.mReader setScanMode:RFIDScanMode];
    [appDelegate.mReader setBarcodeMode:NO isKeyAction:NO];
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
}

#pragma mark - UITableViewDataSource
- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return self.barcodeData.count;
}

#pragma mark - UITableViewDelegate
- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    BarcodeDemoTableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:@"BarcodeCell" forIndexPath:indexPath];
    
    AsBarcodeItem *item = [self.barcodeData objectAtIndex:indexPath.row];
    
    if (item != nil) {
        cell.codeTypeLabel.text = [AsBarcodeType getBarcodeString:item.barcodeType];
        cell.codeIdLabel.text = item.codeId;
        cell.barcodeLabel.text = item.barcode;
    }
    return cell;
}
- (UITableViewCellEditingStyle)tableView:(UITableView *)tableView editingStyleForRowAtIndexPath:(NSIndexPath *)indexPath {
    return UITableViewCellEditingStyleDelete;
}

#pragma mark - UIPickerViewDataSource
- (NSInteger)numberOfComponentsInPickerView:(UIPickerView *)pickerView
{
    return 1;
}
- (NSInteger)pickerView:(UIPickerView *)pickerView numberOfRowsInComponent:(NSInteger)component
{
    return [self.rescanTimePickerData count];
}

#pragma mark - UIPickerViewDelegate
- (NSString *)pickerView:(UIPickerView *)pickerView titleForRow:(NSInteger)row forComponent:(NSInteger)component
{
    return [self.rescanTimePickerData objectAtIndex:row];
}

#pragma mark - IBActions
- (IBAction)clearBtnTapped:(id)sender {
    [self.barcodeData removeAllObjects];
    [[SystemSetting sharedSystemSetting] setBarcodeData:self.barcodeData];
    [self.tableView reloadData];
    self.totalCountLabel.text = [NSString stringWithFormat:@"%lu", (unsigned long)self.barcodeData.count];
}

- (IBAction)scanStartBtnTapped:(id)sender {
    if (isScanning != YES) {
        [self scanEnabled:YES];
    } else {
        [self scanEnabled:NO];
    }
    
}

- (IBAction)rescanTimeBtnTapped:(id)sender {
    self.pickerView.hidden = NO;
    [self.pickerView selectRow:self.rescanTimeValue inComponent:0 animated:NO];
    self.rescanTimePickerTopView.hidden = NO;
}

- (IBAction)pickerCloseBtnTapped:(id)sender {
    [self hidePickerView];
}

- (IBAction)rescanTimePickerOkBtnTapped:(id)sender {
    NSInteger idx = [self.pickerView selectedRowInComponent:0];
    self.rescanTimeTextField.text = [self.rescanTimePickerData objectAtIndex:idx];
    self.rescanTimeValue = idx;
    [[SystemSetting sharedSystemSetting] setBarcodeReScanTime:(int)self.rescanTimeValue];
    [self hidePickerView];
}

- (IBAction)outputCSVBtnTapped:(id)sender {
    dispatch_async(dispatch_get_main_queue(), ^{
        
        NSInteger rfidCount = 0;
        for(int i = 0; i < self.barcodeData.count; i++) {
            AsBarcodeItem *item = nil;
            item = [self.barcodeData objectAtIndex:i];
            rfidCount += [item.barcode integerValue];
        }
        
        if (rfidCount != 0) {
            if ([CSVData isFileExist:@"barcode.csv"]) {
                [CSVData deleteFileForFileName:@"barcode.csv"];
            }
            //创建csv文件
            [CSVData createCSVWithFileName:@"barcode.csv"];
            [CSVData exportBarCodeCSV:@"barcode.csv" withArray:self.barcodeData];
        } else {
            return;
        }
    });
}


#pragma mark - Getters
- (NSArray *)rescanTimePickerData {
    return @[@"Not Used", @"100 ms", @"200 ms", @"300 ms", @"400 ms", @"500 ms", @"600 ms", @"700 ms",
             @"800 ms", @"900 ms", @"1000 ms", @"1100 ms", @"1200 ms", @"1300 ms",
             @"1500 ms", @"2000 ms", @"2500 ms", @"3000 ms", @"3500 ms", @"5000 ms"];
}

- (NSArray *)rescanTimeData {
    return @[@"0", @"100", @"200", @"300", @"400", @"500",
             @"600", @"700", @"800", @"900", @"1000", @"1100", @"1200", @"1300", @"1500", @"2000", @"2500", @"3000",
             @"3500", @"5000"];
}


#pragma mark - Custom
- (void)hidePickerView {
    self.pickerView.hidden = YES;
    self.rescanTimePickerTopView.hidden = YES;
}

- (void)scanEnabled:(BOOL)enabled {
    if (enabled) {
        [appDelegate.mReader startDecode];
        [self scanElemEnabled:NO];
    } else {
        [appDelegate.mReader stopSync];
        [self scanElemEnabled:YES];
    }
}

- (void)scanElemEnabled:(BOOL)enabled {
    if (enabled) {
        isScanning = NO;
        
        [self.startScanBtn setTitle:@"Start Scan" forState:UIControlStateNormal];
        
        for (UIControl *control in self.toDisableElems) {
            [control setEnabled:YES];
        }
        self.outputBtn.enabled = YES;
    } else {
        isScanning = YES;
        
        [self.startScanBtn setTitle:@"Stop Scan" forState:UIControlStateNormal];
        
        for (UIControl *control in self.toDisableElems) {
            [control setEnabled:NO];
        }
        self.outputBtn.enabled = NO;
    }
}

#pragma mark - Reader Event Handler Methods
- (BOOL)onAsReaderTriggerKeyEvent:(BOOL)status{
    isScanning = status;
    if(isScanning){
        [self scanElemEnabled:NO];
    }
    return YES;
}
- (BOOL)onAsReaderLeftModeKeyEvent:(BOOL)status{
    if(appDelegate.mReader.isRFIDModule&&appDelegate.mReader.isBarcodeModule){
        return YES;
    }
    return NO;
}
- (BOOL)onAsReaderRightModeKeyEvent:(BOOL)status{
    if(appDelegate.mReader.isRFIDModule&&appDelegate.mReader.isBarcodeModule){
        return YES;
    }
    return NO;
}
- (void)updateDeviceState:(ResultCode)error {
    if (error != ResultNoError) {
        UIAlertView *alert = [[UIAlertView alloc]initWithTitle:@"Error" message:@"Failed to get response. Try again." delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil];
        [alert show];
        [self.navigationController popViewControllerAnimated:YES];
    }
}

- (void)readTag:(NSString *)tag rssi:(float)rssi phase:(float)phase {
    
}

- (void)changedActionState:(CommandType)action resultCode:(NSInteger)resultCode {
    if(action == CommandStop)
    {
        int rescanTime = [[self.rescanTimeData objectAtIndex:self.rescanTimeValue] intValue];
        if (rescanTime > 0&&isScanning) {
            dispatch_after(dispatch_time(DISPATCH_TIME_NOW, rescanTime / 1000.00 * NSEC_PER_SEC), dispatch_get_main_queue(), ^(void){
                if (isScanning)
                {
                    [appDelegate.mReader startDecode];
                }
            });
        } else {
            [self scanElemEnabled:YES];
        }
    }
}

- (void)detectBarcode:(BarcodeType)barcodeType codeId:(NSString *)codeId barcode:(NSString *)barcode {
    if (barcodeType != BarcodeTypeNoRead) {
        AsBarcodeItem *item = [[AsBarcodeItem alloc] init];
        item.barcodeType = barcodeType;
        item.codeId = codeId;
        item.barcode = barcode;
        NSDate *timeDate = [NSDate date];
        NSDateFormatter *formatter = [[NSDateFormatter alloc] init];
        [formatter setDateStyle:NSDateFormatterMediumStyle];
        [formatter setTimeStyle:NSDateFormatterShortStyle];
        [formatter setDateFormat:@"YYYY/MM/dd HH:mm:ss"];
        NSString *dateTime = [formatter stringFromDate:timeDate];
        item.dateTime = dateTime;
        [self.barcodeData addObject:item];
        [[SystemSetting sharedSystemSetting] setBarcodeData:self.barcodeData];
        [self.tableView reloadData];
        
        [self.tableView scrollToRowAtIndexPath:[NSIndexPath indexPathForRow:self.barcodeData.count - 1 inSection:0] atScrollPosition:UITableViewScrollPositionBottom animated:YES];
        self.totalCountLabel.text = [NSString stringWithFormat:@"%lu", (unsigned long)self.barcodeData.count];
    }
}
//- (void)detectBarcode:(BarcodeType)barcodeType codeId:(NSString *)codeId barcodeData:(NSData *)barcodeData{
//}

- (void)accessResult:(ResultCode)error actionState:(CommandType)action epc:(NSString *)epc data:(NSString *)data rssi:(float)rssi phase:(float)phase frequency:(float)frequency {
}

#pragma mark - NSNotification

- (void)AsReaderGUNDisconnected:(NSNotification *)notification {
    dispatch_async(dispatch_get_main_queue(), ^{
        appDelegate.mReader = nil;
        [self.navigationController popToRootViewControllerAnimated:YES];
    });
}

@end
